@@ -1,7 +1,7 @@
 # -----------------------------------------------------------------------------
 # BSD 3-Clause License
 #
-# Copyright (c) 2018-2021, Science and Technology Facilities Council.
+# Copyright (c) 2018-2022, Science and Technology Facilities Council.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -32,6 +32,7 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # -----------------------------------------------------------------------------
 # Author: S. Siso, STFC Daresbury Lab
+# Modified: R. W. Ford, STFC Daresbury Lab
 
 '''Python script intended to be passed to PSyclone's generate()
 function via the -s option.
@@ -51,7 +52,7 @@ def trans(psy):
     schedule = invoke.schedule
 
     print("DSL level view:")
-    schedule.view()
+    print(schedule.view())
 
     print("f2pygen code:")
     print(str(psy.gen))
@@ -61,7 +62,7 @@ def trans(psy):
 
     print("")
     print("Language level view:")
-    schedule.view()
+    print(schedule.view())
 
     fvisitor = FortranWriter()
     print("")
