@@ -42,17 +42,19 @@ from a PSyIR tree. '''
 # pylint: disable=too-many-lines
 from fparser.two import Fortran2003
 
+from psyclone.core import Signature
 from psyclone.errors import InternalError
 from psyclone.psyir.backend.language_writer import LanguageWriter
 from psyclone.psyir.backend.visitor import VisitorError
 from psyclone.psyir.frontend.fparser2 import Fparser2Reader, \
     TYPE_MAP_FROM_FORTRAN
-from psyclone.psyir.nodes import BinaryOperation, CodeBlock, DataNode, \
+from psyclone.psyir.nodes import BinaryOperation, Call, CodeBlock, DataNode, \
     Literal, Operation, Range, Routine, Schedule, UnaryOperation
 from psyclone.psyir.symbols import ArgumentInterface, ArrayType, \
     ContainerSymbol, DataSymbol, DataTypeSymbol, RoutineSymbol, ScalarType, \
     Symbol, SymbolTable, UnknownFortranType, UnknownType, UnresolvedInterface
 
+
 # The list of Fortran instrinsic functions that we know about (and can
 # therefore distinguish from array accesses). These are taken from
 # fparser.
@@ -298,6 +300,36 @@ def add_accessibility_to_unknown_declaration(symbol):
     return "::".join([first_part]+parts[1:])
 
 
+def _validate_named_args(node):
+    '''Utility function that check that all named args occur after all
+    positional args. The check is applicable to Call and Operation
+    nodes. This is a Fortran restriction, not a PSyIR restriction.
+
+    :param node: the node to check.
+    :type node: :py:class:`psyclone.psyir.nodes.Call` or subclass of \
+    :py:class:`psyclone.psyir.nodes.Operation`
+
+    raises TypeError: if the node is not a Call or Operation.
+    raises VisitorError: if the all of the positional arguments are \
+        not before all of the named arguments.
+
+    '''
+    if not isinstance(node, (Call, Operation)):
+        raise TypeError(
+            f"The _validate_named_args utility function expects either a "
+            f"Call or Operation node, but found '{type(node).__name__}'.")
+
+    found_named_arg = False
+    for name in node.argument_names:
+        if found_named_arg and not name:
+            raise VisitorError(
+                f"Fortran expects all named arguments to occur after all "
+                f"positional arguments but this is not the case for "
+                f"{str(node)}")
+        if name:
+            found_named_arg = True
+
+
 class FortranWriter(LanguageWriter):
     # pylint: disable=too-many-public-methods
     '''Implements a PSyIR-to-Fortran back end for PSyIR kernel code (not
@@ -336,6 +368,13 @@ class FortranWriter(LanguageWriter):
         self._reverse_map(self._operator_2_str,
                           Fparser2Reader.nary_operators)
 
+        # Create and store a DependencyTools instance for use when ordering
+        # parameter declarations. Have to import it here as DependencyTools
+        # also uses this Fortran backend.
+        # pylint: disable=import-outside-toplevel
+        from psyclone.psyir.tools import DependencyTools
+        self._dep_tools = DependencyTools(language_writer=self)
+
     @staticmethod
     def _reverse_map(reverse_dict, op_map):
         '''
@@ -659,9 +698,11 @@ class FortranWriter(LanguageWriter):
 
         self._depth += 1
         for member in symbol.datatype.components.values():
-            # We always want to specify the visibility of components within
-            # a derived type.
-            result += self.gen_vardecl(member, include_visibility=True)
+            # We can only specify the visibility of components within
+            # a derived type if the declaration is within the specification
+            # part of a module.
+            result += self.gen_vardecl(member,
+                                       include_visibility=include_visibility)
         self._depth -= 1
 
         result += f"{self._nindent}end type {symbol.name}\n"
@@ -758,6 +799,72 @@ class FortranWriter(LanguageWriter):
             return result
         return ""
 
+    def _gen_parameter_decls(self, symbol_table, is_module_scope=False):
+        ''' Create the declarations of all parameters present in the supplied
+        symbol table. Declarations are ordered so as to satisfy any inter-
+        dependencies between them.
+
+        :param symbol_table: the SymbolTable instance.
+        :type symbol: :py:class:`psyclone.psyir.symbols.SymbolTable`
+        :param bool is_module_scope: whether or not the declarations are in \
+                                     a module scoping unit. Default is False.
+
+        :returns: Fortran code declaring all parameters.
+        :rtype: str
+
+        :raises VisitorError: if there is no way of resolving \
+                              interdependencies between parameter declarations.
+
+        '''
+        declarations = ""
+        local_constants = [sym for sym in symbol_table.local_datasymbols if
+                           sym.is_constant]
+        if not local_constants:
+            return declarations
+
+        # There may be dependencies between these constants so setup a dict
+        # listing the required inputs for each one.
+        decln_inputs = {}
+        for symbol in local_constants:
+            decln_inputs[symbol.name] = set()
+            input_sigs = self._dep_tools.get_input_parameters(
+                symbol.constant_value)
+            # The dependence analysis tools do not include symbols used to
+            # define precision so check for those here.
+            for lit in symbol.constant_value.walk(Literal):
+                if isinstance(lit.datatype.precision, DataSymbol):
+                    input_sigs.append(Signature(lit.datatype.precision.name))
+            # If the precision of the Symbol being declared is itself defined
+            # by a Symbol then include that as an 'input'.
+            if isinstance(symbol.datatype.precision, DataSymbol):
+                input_sigs.append(Signature(symbol.datatype.precision.name))
+            # Remove any 'inputs' that are not local since these do not affect
+            # the ordering of local declarations.
+            for sig in input_sigs:
+                if symbol_table.lookup(sig.var_name) in local_constants:
+                    decln_inputs[symbol.name].add(sig)
+        # We now iterate over the declarations, declaring those that have their
+        # inputs satisfied. Creating a declaration for a given symbol removes
+        # that symbol as a dependence from any outstanding declarations.
+        declared = set()
+        while local_constants:
+            for symbol in local_constants[:]:
+                inputs = decln_inputs[symbol.name]
+                if inputs.issubset(declared):
+                    # All inputs are satisfied so this declaration can be added
+                    declared.add(Signature(symbol.name))
+                    local_constants.remove(symbol)
+                    declarations += self.gen_vardecl(
+                        symbol, include_visibility=is_module_scope)
+                    break
+            else:
+                # We looped through all of the variables remaining to be
+                # declared and none had their dependencies satisfied.
+                raise VisitorError(
+                    f"Unable to satisfy dependencies for the declarations of "
+                    f"{[sym.name for sym in local_constants]}")
+        return declarations
+
     def gen_decls(self, symbol_table, is_module_scope=False):
         '''Create and return the Fortran declarations for the supplied
         SymbolTable.
@@ -837,7 +944,11 @@ class FortranWriter(LanguageWriter):
         # variable declarations. As a convention, this method also
         # declares any argument variables before local variables.
 
-        # 1: Argument variable declarations
+        # 1: Local constants.
+        declarations += self._gen_parameter_decls(symbol_table,
+                                                  is_module_scope)
+
+        # 2: Argument variable declarations
         if symbol_table.argument_datasymbols and is_module_scope:
             raise VisitorError(
                 f"Arguments are not allowed in this context but this symbol "
@@ -848,13 +959,6 @@ class FortranWriter(LanguageWriter):
             declarations += self.gen_vardecl(
                 symbol, include_visibility=is_module_scope)
 
-        # 2: Local constants.
-        local_constants = [sym for sym in symbol_table.local_datasymbols if
-                           sym.is_constant]
-        for symbol in local_constants:
-            declarations += self.gen_vardecl(
-                symbol, include_visibility=is_module_scope)
-
         # 3: Derived-type declarations. These must come before any declarations
         #    of symbols of these types.
         for symbol in symbol_table.local_datatypesymbols:
@@ -1012,10 +1116,18 @@ class FortranWriter(LanguageWriter):
         # variables at the routine level scope. For this reason, at this
         # point we have to unify all declarations and resolve possible name
         # clashes that appear when merging the scopes.
-        whole_routine_scope = SymbolTable(node)
+        whole_routine_scope = SymbolTable()
 
+        own_symbol = node.symbol_table.lookup_with_tag("own_routine_symbol")
         for schedule in node.walk(Schedule):
             for symbol in schedule.symbol_table.symbols[:]:
+
+                # We don't need to add the Symbol representing this Routine to
+                # the top level symbol table because in Fortran it is already
+                # implicitly declared by the subroutine statement.
+                if symbol is own_symbol and isinstance(symbol, RoutineSymbol):
+                    continue
+
                 try:
                     whole_routine_scope.add(symbol)
                 except KeyError:
@@ -1024,6 +1136,10 @@ class FortranWriter(LanguageWriter):
                     schedule.symbol_table.rename_symbol(symbol, new_name)
                     whole_routine_scope.add(symbol)
 
+        # Replace the symbol table
+        node.symbol_table.detach()
+        whole_routine_scope.attach(node)
+
         # Generate module imports
         imports = ""
         for symbol in whole_routine_scope.containersymbols:
@@ -1073,8 +1189,14 @@ class FortranWriter(LanguageWriter):
         :rtype: str
 
         '''
+        _validate_named_args(node)
+
         lhs = self._visit(node.children[0])
         rhs = self._visit(node.children[1])
+        if node.argument_names[0]:
+            lhs = f"{node.argument_names[0]}={lhs}"
+        if node.argument_names[1]:
+            rhs = f"{node.argument_names[1]}={rhs}"
         try:
             fort_oper = self.get_operator(node.operator)
             if self.is_intrinsic(fort_oper):
@@ -1118,9 +1240,15 @@ class FortranWriter(LanguageWriter):
         :raises VisitorError: if an unexpected N-ary operator is found.
 
         '''
+        _validate_named_args(node)
+
         arg_list = []
-        for child in node.children:
-            arg_list.append(self._visit(child))
+        for idx, child in enumerate(node.children):
+            if node.argument_names[idx]:
+                arg_list.append(
+                    f"{node.argument_names[idx]}={self._visit(child)}")
+            else:
+                arg_list.append(self._visit(child))
         try:
             fort_oper = self.get_operator(node.operator)
             return f"{fort_oper}(" + ", ".join(arg_list) + ")"
@@ -1314,7 +1442,11 @@ class FortranWriter(LanguageWriter):
             fort_oper = self.get_operator(node.operator)
             if self.is_intrinsic(fort_oper):
                 # This is a unary intrinsic function.
-                return f"{fort_oper}({content})"
+                if node.argument_names[0]:
+                    result = f"{fort_oper}({node.argument_names[0]}={content})"
+                else:
+                    result = f"{fort_oper}({content})"
+                return result
             # It's not an intrinsic function so we need to consider the
             # parent node. If that is a UnaryOperation or a BinaryOperation
             # such as '-' or '**' then we need parentheses. This ensures we
@@ -1483,10 +1615,15 @@ class FortranWriter(LanguageWriter):
         :rtype: str
 
         '''
+        _validate_named_args(node)
 
         result_list = []
-        for child in node.children:
-            result_list.append(self._visit(child))
+        for idx, child in enumerate(node.children):
+            if node.argument_names[idx]:
+                result_list.append(
+                    f"{node.argument_names[idx]}={self._visit(child)}")
+            else:
+                result_list.append(self._visit(child))
         args = ", ".join(result_list)
         if not node.parent or isinstance(node.parent, Schedule):
             return f"{self._nindent}call {node.routine.name}({args})\n"
