@@ -31,15 +31,15 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # -----------------------------------------------------------------------------
 # Author: S. Siso, STFC Daresbury Lab
-# Modifications: A. R. Porter, STFC Daresbury Lab
+# Modifications: A. R. Porter and N. Nobre, STFC Daresbury Lab
 # -----------------------------------------------------------------------------
 
 ''' This module provides the PSyIR Fortran front-end.'''
 
-import six
 from fparser.common.readfortran import FortranStringReader
 from fparser.two import Fortran2003
 from fparser.two.parser import ParserFactory
+from fparser.two.symbol_table import SYMBOL_TABLES
 from fparser.two.utils import NoMatchError
 from psyclone.psyir.frontend.fparser2 import Fparser2Reader
 from psyclone.psyir.nodes import Schedule, Assignment
@@ -58,6 +58,7 @@ class FortranReader(object):
         if not self._parser:
             self._parser = ParserFactory().create(std="f2008")
         self._processor = Fparser2Reader()
+        SYMBOL_TABLES.clear()
 
     def psyir_from_source(self, source_code):
         ''' Generate the PSyIR tree representing the given Fortran source code.
@@ -68,6 +69,7 @@ class FortranReader(object):
         :rtype: :py:class:`psyclone.psyir.nodes.Node`
 
         '''
+        SYMBOL_TABLES.clear()
         string_reader = FortranStringReader(source_code)
         parse_tree = self._parser(string_reader)
         psyir = self._processor.generate_psyir(parse_tree)
@@ -93,20 +95,24 @@ class FortranReader(object):
             to be brought into scope).
         '''
         if not isinstance(symbol_table, SymbolTable):
-            raise TypeError("Must be supplied with a valid SymbolTable but got"
-                            " '{0}'".format(type(symbol_table).__name__))
+            raise TypeError(f"Must be supplied with a valid SymbolTable but "
+                            f"got '{type(symbol_table).__name__}'")
 
         try:
             parse_tree = Fortran2003.Expr(source_code)
         except NoMatchError as err:
-            six.raise_from(
-                ValueError("Supplied source does not represent a Fortran "
-                           "expression: '{0}'".format(source_code)), err)
+            raise ValueError(
+                f"Supplied source does not represent a Fortran "
+                f"expression: '{source_code}'") from err
 
         # Create a fake sub-tree connected to the supplied symbol table so
         # that we can process the expression and lookup any symbols that it
-        # references.
-        fake_parent = Schedule(symbol_table=symbol_table)
+        # references. We provide the SymbolTable directly to the private
+        # attribute to avoid the symbol table's node link to connect to the
+        # new Schedule and therfore stealing it from its original scope.
+        # pylint: disable=protected-access
+        fake_parent = Schedule()
+        fake_parent._symbol_table = symbol_table
         fake_parent.addchild(Assignment())
 
         try:
@@ -114,11 +120,10 @@ class FortranReader(object):
             # created as the parent.
             self._processor.process_nodes(fake_parent[0], [parse_tree])
         except SymbolError as err:
-            six.raise_from(
-                SymbolError("Expression '{0}' contains symbols which are not "
-                            "present in any symbol table and there are no "
-                            "wildcard imports which might be bringing them "
-                            "into scope.".format(source_code)), err)
+            raise SymbolError(f"Expression '{source_code}' contains symbols "
+                              f"which are not present in any symbol table and "
+                              f"there are no wildcard imports which might be "
+                              f"bringing them into scope.") from err
         return fake_parent[0].children[0].detach()
 
     def psyir_from_statement(self, source_code, symbol_table):
@@ -153,7 +158,7 @@ class FortranReader(object):
         # Create a fake sub-tree connected to the supplied symbol table so
         # that we can process the statement and lookup any symbols that it
         # references.
-        fake_parent = Schedule(symbol_table=symbol_table)
+        fake_parent = Schedule(symbol_table=symbol_table.deep_copy())
 
         try:
             # Process the statement, giving the Schedule we've just
@@ -174,7 +179,10 @@ class FortranReader(object):
 
         :returns: PSyIR representing the provided Fortran file.
         :rtype: :py:class:`psyclone.psyir.nodes.Node`
+
         '''
+        SYMBOL_TABLES.clear()
+
         # Note that this is the main performance hotspot in PSyclone, taking
         # more than 90% of the runtime in some cases. Therefore this is a good
         # place to implement caching in order to avoid repeating parsing steps
