@@ -31,7 +31,7 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 # -----------------------------------------------------------------------------
-# Authors R. W. Ford, A. R. Porter and S. Siso, STFC Daresbury Lab
+# Authors R. W. Ford, A. R. Porter, S. Siso and N. Nobre, STFC Daresbury Lab
 #         A. B. G. Chalk, STFC Daresbury Lab
 #         I. Kavcic,    Met Office
 #         C.M. Maynard, Met Office / University of Reading
@@ -472,17 +472,16 @@ class OMPParallelDirective(OMPRegionDirective):
             name = call.reduction_arg.name
             if name in names:
                 raise GenerationError(
-                    "Reduction variables can only be used once in an invoke. "
-                    "'{0}' is used multiple times, please use a different "
-                    "reduction variable".format(name))
+                    f"Reduction variables can only be used once in an invoke. "
+                    f"'{name}' is used multiple times, please use a different "
+                    f"reduction variable")
             else:
                 names.append(name)
 
         zero_reduction_variables(calls, parent)
 
         parent.add(DirectiveGen(parent, "omp", "begin", "parallel",
-                                "default(shared), private({0})".
-                                format(private_str)))
+                                f"default(shared), private({private_str})"))
 
         if reprod_red_call_list:
             # add in a local thread index
@@ -527,7 +526,7 @@ class OMPParallelDirective(OMPRegionDirective):
         private_str = ",".join(private_list)
 
         if private_str:
-            result = "{0}, private({1})".format(result, private_str)
+            result = f"{result}, private({private_str})"
         return result
 
     def end_string(self):
@@ -561,8 +560,8 @@ class OMPParallelDirective(OMPRegionDirective):
             for variable_name in call.local_vars():
                 if variable_name == "":
                     raise InternalError(
-                        "call '{0}' has a local variable but its "
-                        "name is not set.".format(call.name))
+                        f"call '{call.name}' has a local variable but its "
+                        f"name is not set.")
                 result.add(variable_name.lower())
 
         # Now determine scalar variables that must be private:
@@ -742,9 +741,8 @@ class OMPTaskloopDirective(OMPRegionDirective):
         '''
         # It is only at the point of code generation that we can check for
         # correctness (given that we don't mandate the order that a user
-        # can apply transformations to the code). A taskloop
-        # directive, we must have an OMPSerialDirective as an
-        # ancestor back up the tree.
+        # can apply transformations to the code). A taskloop directive, we must
+        # have an OMPSerialDirective as an ancestor back up the tree.
         if not self.ancestor(OMPSerialDirective):
             raise GenerationError(
                 "OMPTaskloopDirective must be inside an OMP Serial region "
@@ -779,9 +777,9 @@ class OMPTaskloopDirective(OMPRegionDirective):
         # Find the specified clauses
         clause_list = []
         if self._grainsize is not None:
-            clause_list.append("grainsize({0})".format(self._grainsize))
+            clause_list.append(f"grainsize({self._grainsize})")
         if self._num_tasks is not None:
-            clause_list.append("num_tasks({0})".format(self._num_tasks))
+            clause_list.append(f"num_tasks({self._num_tasks})")
         if self._nogroup:
             clause_list.append("nogroup")
         # Generate the string containing the required clauses
@@ -849,6 +847,23 @@ class OMPDoDirective(OMPRegionDirective):
         super(OMPDoDirective, self).__init__(children=children,
                                              parent=parent)
 
+    def __eq__(self, other):
+        '''
+        Checks whether two nodes are equal. Two OMPDoDirective nodes are equal
+        if they have the same schedule, the same reproducible reduction option
+        (and the inherited equality is True).
+
+        :param object other: the object to check equality to.
+
+        :returns: whether other is equal to self.
+        :rtype: bool
+        '''
+        is_eq = super().__eq__(other)
+        is_eq = is_eq and self.omp_schedule == other.omp_schedule
+        is_eq = is_eq and self.reprod == other.reprod
+
+        return is_eq
+
     def node_str(self, colour=True):
         '''
         Returns the name of this node with (optional) control codes
@@ -860,10 +875,10 @@ class OMPDoDirective(OMPRegionDirective):
         :rtype: str
         '''
         if self.reductions():
-            reprod = "reprod={0}".format(self._reprod)
+            reprod = f"reprod={self._reprod}"
         else:
             reprod = ""
-        return "{0}[{1}]".format(self.coloured_name(colour), reprod)
+        return f"{self.coloured_name(colour)}[{reprod}]"
 
     def _reduction_string(self):
         ''' Return the OMP reduction information as a string '''
@@ -871,10 +886,19 @@ class OMPDoDirective(OMPRegionDirective):
         for reduction_type in AccessType.get_valid_reduction_modes():
             reductions = self._get_reductions_list(reduction_type)
             for reduction in reductions:
-                reduction_str += ", reduction({0}:{1})".format(
-                    OMP_OPERATOR_MAPPING[reduction_type], reduction)
+                reduction_str += (f", reduction("
+                                  f"{OMP_OPERATOR_MAPPING[reduction_type]}:"
+                                  f"{reduction})")
         return reduction_str
 
+    @property
+    def omp_schedule(self):
+        '''
+        :returns: the omp_schedule for this object.
+        :rtype: str
+        '''
+        return self._omp_schedule
+
     @property
     def reprod(self):
         ''' returns whether reprod has been set for this object or not '''
@@ -913,16 +937,15 @@ class OMPDoDirective(OMPRegionDirective):
         '''
         if len(self.dir_body.children) != 1:
             raise GenerationError(
-                "An {0} can only be applied to a single loop "
-                "but this Node has {1} children: {2}".
-                format(type(self).__name__, len(self.dir_body.children),
-                       self.dir_body.children))
+                f"An {type(self).__name__} can only be applied to a single "
+                f"loop but this Node has {len(self.dir_body.children)} "
+                f"children: {self.dir_body.children}")
 
         if not isinstance(self.dir_body[0], Loop):
             raise GenerationError(
-                "An {0} can only be applied to a loop "
-                "but this Node has a child of type '{1}'".format(
-                    type(self).__name__, type(self.dir_body[0]).__name__))
+                f"An {type(self).__name__} can only be applied to a loop but "
+                f"this Node has a child of type "
+                f"'{type(self.dir_body[0]).__name__}'")
 
     def gen_code(self, parent):
         '''
@@ -946,8 +969,7 @@ class OMPDoDirective(OMPRegionDirective):
         # As we're a loop we don't specify the scope
         # of any variables so we don't have to generate the
         # list of private variables
-        options = "schedule({0})".format(self._omp_schedule) + \
-                  local_reduction_string
+        options = f"schedule({self._omp_schedule}){local_reduction_string}"
         parent.add(DirectiveGen(parent, "omp", "begin", "do", options))
 
         for child in self.children:
@@ -967,7 +989,7 @@ class OMPDoDirective(OMPRegionDirective):
         :rtype: str
 
         '''
-        return "omp do schedule({0})".format(self._omp_schedule)
+        return f"omp do schedule({self._omp_schedule})"
 
     def end_string(self):
         '''Returns the end (or closing) statement of this directive, i.e.
@@ -1005,9 +1027,8 @@ class OMPParallelDoDirective(OMPParallelDirective, OMPDoDirective):
         zero_reduction_variables(calls, parent)
         private_str = ",".join(self._get_private_list())
         parent.add(DirectiveGen(parent, "omp", "begin", "parallel do",
-                                "default(shared), private({0}), "
-                                "schedule({1})".
-                                format(private_str, self._omp_schedule) +
+                                f"default(shared), private({private_str}), "
+                                f"schedule({self._omp_schedule})" +
                                 self._reduction_string()))
         for child in self.children:
             child.gen_code(parent)
@@ -1027,9 +1048,8 @@ class OMPParallelDoDirective(OMPParallelDirective, OMPDoDirective):
 
         '''
         private_str = ",".join(self._get_private_list())
-        return ("omp parallel do default(shared), private({0}), "
-                "schedule({1})".format(private_str, self._omp_schedule) +
-                self._reduction_string())
+        return (f"omp parallel do default(shared), private({private_str}), "
+                f"schedule({self._omp_schedule})" + self._reduction_string())
 
     def end_string(self):
         '''
@@ -1092,6 +1112,22 @@ class OMPLoopDirective(OMPRegionDirective):
         self._collapse = None
         self.collapse = collapse  # Use setter with error checking
 
+    def __eq__(self, other):
+        '''
+        Checks whether two nodes are equal. Two OMPLoopDirective nodes are
+        equal if they have the same collapse status and the inherited
+        equality is true.
+
+        :param object other: the object to check equality to.
+
+        :returns: whether other is equal to self.
+        :rtype: bool
+        '''
+        is_eq = super().__eq__(other)
+        is_eq = is_eq and self.collapse == other.collapse
+
+        return is_eq
+
     @property
     def collapse(self):
         '''
@@ -1114,15 +1150,13 @@ class OMPLoopDirective(OMPRegionDirective):
         '''
         if value is not None and not isinstance(value, int):
             raise TypeError(
-                "The OMPLoopDirective collapse clause must be a positive "
-                "integer or None, but value '{0}' has been given."
-                "".format(value))
+                f"The OMPLoopDirective collapse clause must be a positive "
+                f"integer or None, but value '{value}' has been given.")
 
         if value is not None and value <= 0:
             raise ValueError(
-                "The OMPLoopDirective collapse clause must be a positive "
-                "integer or None, but value '{0}' has been given."
-                "".format(value))
+                f"The OMPLoopDirective collapse clause must be a positive "
+                f"integer or None, but value '{value}' has been given.")
 
         self._collapse = value
 
@@ -1137,7 +1171,7 @@ class OMPLoopDirective(OMPRegionDirective):
         '''
         text = self.coloured_name(colour)
         if self._collapse:
-            text += "[collapse={0}]".format(str(self._collapse))
+            text += f"[collapse={self._collapse}]"
         else:
             text += "[]"
         return text
@@ -1153,7 +1187,7 @@ class OMPLoopDirective(OMPRegionDirective):
         '''
         string = "omp loop"
         if self._collapse:
-            string += " collapse({0})".format(str(self._collapse))
+            string += f" collapse({self._collapse})"
         return string
 
     def end_string(self):
@@ -1184,14 +1218,13 @@ class OMPLoopDirective(OMPRegionDirective):
         '''
         if len(self.dir_body.children) != 1:
             raise GenerationError(
-                "OMPLoopDirective must have exactly one child in its "
-                "associated schedule but found {0}.".format(
-                    self.dir_body.children))
+                f"OMPLoopDirective must have exactly one child in its "
+                f"associated schedule but found {self.dir_body.children}.")
 
         if not isinstance(self.dir_body.children[0], Loop):
             raise GenerationError(
-                "OMPLoopDirective must have a Loop as child of its associated "
-                "schedule but found '{0}'.".format(self.dir_body.children[0]))
+                f"OMPLoopDirective must have a Loop as child of its associated"
+                f" schedule but found '{self.dir_body.children[0]}'.")
 
         if not self.ancestor((OMPTargetDirective, OMPParallelDirective)):
             # Also omp teams or omp threads regions but these are not supported
@@ -1207,12 +1240,11 @@ class OMPLoopDirective(OMPRegionDirective):
             for depth in range(self._collapse):
                 if not isinstance(cursor, Loop):
                     raise GenerationError(
-                        "OMPLoopDirective must have as many immediately nested"
-                        " loops as the collapse clause specifies but '{0}' "
-                        "has a collpase={1} and the nested statement at depth "
-                        "{2} is a {3} rather than a Loop."
-                        "".format(self, self._collapse, depth,
-                                  type(cursor).__name__))
+                        f"OMPLoopDirective must have as many immediately "
+                        f"nested loops as the collapse clause specifies but "
+                        f"'{self}' has a collapse={self._collapse} and the "
+                        f"nested statement at depth {depth} is a "
+                        f"{type(cursor).__name__} rather than a Loop.")
                 cursor = cursor.loop_body.children[0]
 
         super(OMPLoopDirective, self).validate_global_constraints()
