@@ -1,7 +1,7 @@
 # -----------------------------------------------------------------------------
 # BSD 3-Clause License
 #
-# Copyright (c) 2020-2021, Science and Technology Facilities Council.
+# Copyright (c) 2020-2022, Science and Technology Facilities Council.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -31,7 +31,7 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 # -----------------------------------------------------------------------------
-# Author R. W. Ford, STFC Daresbury Lab
+# Author R. W. Ford and N. Nobre, STFC Daresbury Lab
 # Modified by J. Henrichs, Bureau of Meteorology
 
 '''Module providing a transformation from a PSyIR Array Range to a
@@ -68,7 +68,7 @@ class ArrayRange2LoopTrans(Transformation):
     >>> from psyclone.psyir.transformations import ArrayRange2LoopTrans, \
     >>>     TransformationError
     >>>
-    >>> schedule.view()
+    >>> print(schedule.view())
     >>> trans = ArrayRange2LoopTrans()
     >>> for assignment in schedule.walk(Assignment):
     >>>     while True:
@@ -76,7 +76,7 @@ class ArrayRange2LoopTrans(Transformation):
     >>>             trans.apply(assignment)
     >>>         except TransformationError:
     >>>             break
-    >>> schedule.view()
+    >>> print(schedule.view())
 
     '''
 
@@ -112,44 +112,40 @@ class ArrayRange2LoopTrans(Transformation):
         # pylint: disable=too-many-branches
         if not isinstance(array1, ArrayReference):
             raise TypeError(
-                "The first argument to the same_range() method should be an "
-                "ArrayReference but found '{0}'.".format(
-                    type(array1).__name__))
+                f"The first argument to the same_range() method should be an "
+                f"ArrayReference but found '{type(array1).__name__}'.")
         if not isinstance(idx1, int):
             raise TypeError(
-                "The second argument to the same_range() method should be an "
-                "int but found '{0}'.".format(type(idx1).__name__))
+                f"The second argument to the same_range() method should be an "
+                f"int but found '{type(idx1).__name__}'.")
         if not isinstance(array2, ArrayReference):
             raise TypeError(
-                "The third argument to the same_range() method should be an "
-                "ArrayReference but found '{0}'.".format(
-                    type(array2).__name__))
+                f"The third argument to the same_range() method should be an "
+                f"ArrayReference but found '{type(array2).__name__}'.")
         if not isinstance(idx2, int):
             raise TypeError(
-                "The fourth argument to the same_range() method should be an "
-                "int but found '{0}'.".format(type(idx2).__name__))
+                f"The fourth argument to the same_range() method should be an "
+                f"int but found '{type(idx2).__name__}'.")
         if not idx1 < len(array1.children):
             raise IndexError(
-                "The value of the second argument to the same_range() method "
-                "'{0}' should be less than the number of dimensions '{1}' in "
-                "the associated array 'array1'."
-                "".format(idx1, len(array1.children)))
+                f"The value of the second argument to the same_range() method "
+                f"'{idx1}' should be less than the number of dimensions "
+                f"'{len(array1.children)}' in the associated array 'array1'.")
         if not idx2 < len(array2.children):
             raise IndexError(
-                "The value of the fourth argument to the same_range() method "
-                "'{0}' should be less than the number of dimensions '{1}' in "
-                "the associated array 'array2'."
-                "".format(idx2, len(array2.children)))
+                f"The value of the fourth argument to the same_range() method "
+                f"'{idx2}' should be less than the number of dimensions "
+                f"'{len(array2.children)}' in the associated array 'array2'.")
         if not isinstance(array1.children[idx1], Range):
             raise TypeError(
-                "The child of the first array argument at the specified index "
-                "({0}) should be a Range node, but found '{1}'."
-                "".format(idx1, type(array1.children[idx1]).__name__))
+                f"The child of the first array argument at the specified index"
+                f" ({idx1}) should be a Range node, but found "
+                f"'{type(array1.children[idx1]).__name__}'.")
         if not isinstance(array2.children[idx2], Range):
             raise TypeError(
-                "The child of the second array argument at the specified "
-                "index ({0}) should be a Range node, but found '{1}'."
-                "".format(idx2, type(array2.children[idx2]).__name__))
+                f"The child of the second array argument at the specified "
+                f"index ({idx2}) should be a Range node, but found "
+                f"'{type(array2.children[idx2]).__name__}'.")
 
         range1 = array1.children[idx1]
         range2 = array2.children[idx2]
@@ -278,22 +274,22 @@ class ArrayRange2LoopTrans(Transformation):
         '''
         if not isinstance(node, Assignment):
             raise TransformationError(
-                "Error in {0} transformation. The supplied node argument "
-                "should be a PSyIR Assignment, but found '{1}'."
-                "".format(self.name, type(node).__name__))
+                f"Error in {self.name} transformation. The supplied node "
+                f"argument should be a PSyIR Assignment, but found "
+                f"'{type(node).__name__}'.")
 
         if not isinstance(node.lhs, ArrayReference):
             raise TransformationError(
-                "Error in {0} transformation. The lhs of the supplied "
-                "Assignment node should be a PSyIR ArrayReference, but "
-                "found '{1}'.".format(self.name, type(node.lhs).__name__))
+                f"Error in {self.name} transformation. The lhs of the "
+                f"supplied Assignment node should be a PSyIR ArrayReference, "
+                f"but found '{type(node.lhs).__name__}'.")
 
         if not [dim for dim in node.lhs.children if isinstance(dim, Range)]:
             raise TransformationError(
-                "Error in {0} transformation. The lhs of the supplied "
-                "Assignment node should be a PSyIR ArrayReference with at "
-                "least one of its dimensions being a Range, but found None "
-                "in '{1}'.".format(self.name, str(node.lhs)))
+                f"Error in {self.name} transformation. The lhs of the supplied"
+                f" Assignment node should be a PSyIR ArrayReference with at "
+                f"least one of its dimensions being a Range, but found None "
+                f"in '{node.lhs}'.")
 
         # If an operator on the rhs only returns an array then we are
         # not able to turn the assignment into an explicit loop. At
@@ -306,10 +302,9 @@ class ArrayRange2LoopTrans(Transformation):
         if [operation for operation in node.rhs.walk(Operation)
                 if operation.operator in [BinaryOperation.Operator.MATMUL]]:
             raise TransformationError(
-                "Error in {0} transformation. The rhs of the supplied "
-                "Assignment node '{1}' contains the MATMUL operator which "
-                "can't be performed elementwise."
-                "".format(self.name, str(node.rhs)))
+                f"Error in {self.name} transformation. The rhs of the "
+                f"supplied Assignment node '{node.rhs}' contains the MATMUL "
+                f"operator which can't be performed elementwise.")
 
         # Find the outermost range for the array on the lhs of the
         # assignment and save its index.
@@ -334,14 +329,13 @@ class ArrayRange2LoopTrans(Transformation):
                         # can't safely replace this range with a
                         # loop iterator.
                         raise TransformationError(
-                            "The ArrayRange2LoopTrans transformation only "
-                            "supports ranges that are known to be the "
-                            "same as each other but array access '{0}' "
-                            "dimension {1} and '{2}' dimension {3} are "
-                            "either different or can't be determined in the "
-                            "assignment '{4}'."
-                            "".format(node.lhs.name, lhs_index, array.name,
-                                      idx, str(node)))
+                            f"The ArrayRange2LoopTrans transformation only "
+                            f"supports ranges that are known to be the "
+                            f"same as each other but array access "
+                            f"'{node.lhs.name}' dimension {lhs_index} and "
+                            f"'{array.name}' dimension {idx} are either "
+                            f"different or can't be determined in the "
+                            f"assignment '{node}'.")
                     break
 
 
