@@ -31,7 +31,7 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 # -----------------------------------------------------------------------------
-# Author: A. R. Porter, STFC Daresbury Lab
+# Authors: A. R. Porter and N. Nobre, STFC Daresbury Lab
 
 '''This module contains the base class LoopTrans. All transformations which
    act on a loop sub-class this one.
@@ -87,23 +87,22 @@ class LoopTrans(Transformation):
         # pylint: disable=too-many-branches
         if not isinstance(node, Loop):
             raise TransformationError(
-                "Target of {0} transformation must be a sub-class of Loop "
-                "but got '{1}'".format(self.name, type(node).__name__))
+                f"Target of {self.name} transformation must be a sub-class of "
+                f"Loop but got '{type(node).__name__}'")
 
         # The loop must be fully-formed.
         if len(node.children) != 4:
             raise TransformationError(
-                "Error in {0} transformation. The target loop "
-                "must have four children but found: {1}.".format(
-                    self.name,
-                    [type(child).__name__ for child in node.children]))
+                f"Error in {self.name} transformation. The target loop "
+                f"must have four children but found: "
+                f"{[type(child).__name__ for child in node.children]}.")
 
         if not options:
             options = {}
         if not isinstance(options, dict):
             raise TransformationError(
-                "Transformation validate method 'options' argument must be a "
-                "dictionary but found '{0}'.".format(type(options).__name__))
+                f"Transformation validate method 'options' argument must be a "
+                f"dictionary but found '{type(options).__name__}'.")
 
         # Check that the proposed region contains only supported node types
         if options.get("node-type-check", True):
@@ -114,17 +113,15 @@ class LoopTrans(Transformation):
             for item in flat_list:
                 if isinstance(item, self.excluded_node_types):
                     raise TransformationError(
-                        "Nodes of type '{0}' cannot be enclosed by a {1} "
-                        "transformation".format(type(item).__name__,
-                                                self.name))
+                        f"Nodes of type '{type(item).__name__}' cannot be "
+                        f"enclosed by a {self.name} transformation")
 
         # A 'null' loop is one which exists in the PSyIR hierarchy (mainly for
         # halo-exchange logic) but does *not* correspond to an actual loop
         # in the code that is generated for the PSy layer.
         if node.loop_type == 'null':
             raise TransformationError(
-                "Cannot apply a {0} transformation to a 'null' loop.".format(
-                    self.name))
+                f"Cannot apply a {self.name} transformation to a 'null' loop.")
 
     @property
     def name(self):
