@@ -31,7 +31,7 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 # -----------------------------------------------------------------------------
-# Author: R. W. Ford and S. Siso, STFC Daresbury Lab
+# Authors: R. W. Ford, S. Siso and N. Nobre, STFC Daresbury Lab
 
 '''This module provides the TransformationError class.
 '''
@@ -46,7 +46,7 @@ class TransformationError(PSycloneError):
     def __init__(self, value):
         PSycloneError.__init__(self, value)
         self.value = LazyString(
-            lambda: "Transformation Error: {0}".format(value))
+            lambda: f"Transformation Error: {value}")
 
 
 # TODO #1280: This currently causes 'more than one target for cross-reference'
